/*
 * The MIT License
 *
 * Copyright (c) 2018, CloudBees, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package jenkins.diagnostics;

import hudson.Extension;
import hudson.model.AdministrativeMonitor;
import hudson.security.Permission;
import jenkins.model.Jenkins;
import jenkins.model.JenkinsLocationConfiguration;
import jenkins.util.UrlHelper;
import org.jenkinsci.Symbol;
import org.kohsuke.accmod.Restricted;
import org.kohsuke.accmod.restrictions.NoExternalUse;

/**
 * Jenkins URL is required for a lot of operations in both core and plugins.
 * There is a default behavior (infer the URL from the request object), but inaccurate in some scenarios.
 * Normally this root URL is set during SetupWizard phase, this monitor is there to ensure that behavior.
 * Potential exceptions are the dev environment, if someone disable the wizard or
 * the administrator put an empty string on the configuration page.
 *
 * @since 2.119
 */
@Extension
@Symbol("rootUrlNotSet")
@Restricted(NoExternalUse.class)
public class RootUrlNotSetMonitor extends AdministrativeMonitor {
    @Override
    public String getDisplayName() {
        return Messages.RootUrlNotSetMonitor_DisplayName();
    }

    @Override
    public boolean isActivated() {
        JenkinsLocationConfiguration loc = JenkinsLocationConfiguration.get();
        return loc.getUrl() == null || !UrlHelper.isValidRootUrl(loc.getUrl());
    }

    // used by jelly to determined if it's a null url or invalid one
    @Restricted(NoExternalUse.class)
    public boolean isUrlNull() {
        JenkinsLocationConfiguration loc = JenkinsLocationConfiguration.get();
        return loc.getUrl() == null;
    }

    @Override
    public Permission getRequiredPermission() {
        return Jenkins.SYSTEM_READ;
    }
}
